# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Properties classes for PH-ScheduleRuleset objects."""

try:
    from typing import Any, Generator
except:
    pass  # IronPython


class ScheduleRulesetPhProperties_FromDictError(Exception):
    def __init__(self, _expected_types, _input_type):
        self.msg = 'Error: Expected type of "{}". Got: {}'.format(
            _expected_types[0], _expected_types[1], _input_type)
        super(ScheduleRulesetPhProperties_FromDictError, self).__init__(self.msg)


class DailyOperationPeriod(object):
    """PH-Style Daily Operating Period"""

    def __init__(self):
        self.name = ''
        self.start_hour = 0
        self.end_hour = 24
        self.operation_fraction = 1.0

    @classmethod
    def from_annual_utilization_factor(cls, _ann_util_fac=1.0, _name=''):
        # type: (float, str) -> DailyOperationPeriod
        op_period = cls()

        op_period.name = _name
        op_period.start_hour = 0
        op_period.end_hour = 24
        op_period.operation_fraction = _ann_util_fac

        return op_period

    @classmethod
    def from_start_end_hours(cls, _start_hr=0, _end_hr=24, _op_frac=1.0, _name=''):
        # type: (float, float, float, str) -> DailyOperationPeriod
        op_period = cls()

        op_period.name = _name
        op_period.start_hour = _start_hr
        op_period.end_hour = _end_hr
        op_period.operation_fraction = _op_frac

        return op_period

    @classmethod
    def from_operating_hours(cls, _op_hours=24, _op_frac=1.0, _name=''):
        # type: (float, float, str) -> DailyOperationPeriod
        op_period = cls()

        op_period.name = _name
        op_period.start_hour = 12.0 - (_op_hours / 2.0)
        op_period.end_hour = 12.0 + (_op_hours / 2.0)
        op_period.operation_fraction = _op_frac

        return op_period

    @property
    def operation_hours(self):
        # type: () -> float
        return self.end_hour - self.start_hour

    @property
    def weighted_operation_hours(self):
        # type: () -> float
        return self.operation_hours * self.operation_fraction

    def to_dict(self):
        # type: () -> dict[str, float]
        d = {}
        d['name'] = self.name
        d['start_hour'] = self.start_hour
        d['end_hour'] = self.end_hour
        d['operation_fraction'] = self.operation_fraction
        return d

    @classmethod
    def from_dict(cls, _dict):
        # type: (dict[str, float]) -> DailyOperationPeriod
        new_op_period = cls()
        new_op_period.name = _dict['name']
        new_op_period.start_hour = _dict['start_hour']
        new_op_period.end_hour = _dict['end_hour']
        new_op_period.operation_fraction = _dict['operation_fraction']
        return new_op_period

    def ToString(self):
        """Overwrite .NET ToString."""
        return self.__repr__()

    def __repr__(self):
        """Properties representation."""
        return '{}: (_operation_hours={}, _operation_fraction={})'.format(
            self.__class__.__name__, self.operation_hours, self.operation_fraction)


class DailyOperatingPeriodCollection(object):
    """A collection of DailyOperatingPeriods"""

    def __init__(self):
        self._collection = []

    def add_period_to_collection(self, _period):
        # type: (DailyOperationPeriod) -> None
        self._collection.append(_period)

    def __nonzero__(self):
        # type: () -> bool
        return bool(self._collection)

    def __bool__(self):
        return self.__nonzero__()

    def __iter__(self):
        # type: () -> Generator[DailyOperationPeriod, None, None]
        for _ in self._collection:
            yield _

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = {}

        periods = []
        for op_period in self._collection:
            periods.append(op_period.to_dict())
        d['collection'] = periods

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> DailyOperatingPeriodCollection
        new_obj = DailyOperatingPeriodCollection()

        for period_dict in _input_dict['collection']:
            new_obj.add_period_to_collection(DailyOperationPeriod.from_dict(period_dict))

        return new_obj


class ScheduleRulesetPhProperties(object):
    """Honeybee-PH ScheduleRulesetPhProperties for logging PH-style schedule data."""

    def __init__(self, _host):
        self._host = _host
        self.id_num = 0
        self.operating_weeks_year = 52
        self.operating_days_wk = 7.0
        self.daily_operating_periods = DailyOperatingPeriodCollection()

    @classmethod
    def from_days_per_week(cls, _days_per_wk, _wks_per_year, _host):
        # type: (float, float, Any) -> ScheduleRulesetPhProperties
        prop_obj = cls(_host)

        prop_obj.operating_days_wk = _days_per_wk
        prop_obj.operating_weeks_year = _wks_per_year

        return prop_obj

    @classmethod
    def from_days_per_year(cls, _days_per_year, _host):
        # type: (float, Any) -> ScheduleRulesetPhProperties
        prop_obj = cls(_host)

        prop_obj.operating_days_wk = (_days_per_year / 365) * 7.0
        prop_obj.operating_weeks_year = 52

        return prop_obj

    @property
    def operating_days_year(self):
        # type: () -> float
        return self.operating_weeks_year * self.operating_days_wk

    @property
    def host(self):
        return self._host

    @property
    def annual_average_operating_fraction(self):
        # type: () -> float
        """Returns the annual average operating (utilization) fraction (0-1)."""

        wtd_total = 0
        for op_period in self.daily_operating_periods:
            wtd_total += op_period.weighted_operation_hours * self.operating_days_year

        annual_operating_hours = self.operating_days_year * 24
        try:
            wtd_annual_avg = wtd_total / annual_operating_hours
        except ZeroDivisionError:
            wtd_annual_avg = 0

        return wtd_annual_avg

    def validate_operating_period_hours(self, _total_period_hours=24.0):
        # type: (float) -> str | None
        """Returns a warning if the total daily operating period hours do not equal the specified _total_period_hours."""

        total_hours = 0
        for op_period in self.daily_operating_periods:
            total_hours += op_period.operation_hours

        if abs(_total_period_hours - total_hours) > 0.001:
            return 'Error: Total Operating Hours={}, not {}?'.format(total_hours, _total_period_hours)
        else:
            return None

    def to_dict(self, abridged=False):
        # type: (bool) -> dict[str, Any]
        d = {}
        if abridged:
            d['type'] = 'ScheduleRulesetPhPropertiesAbridged'
        else:
            d['type'] = 'ScheduleRulesetPhProperties'

        d['id_num'] = self.id_num
        d['operating_weeks_year'] = self.operating_weeks_year
        d['operating_days_wk'] = self.operating_days_wk
        d['operating_periods'] = self.daily_operating_periods.to_dict()

        return {'ph': d}

    @classmethod
    def from_dict(cls, _dict, host):
        # type: (dict, Any) -> ScheduleRulesetPhProperties
        valid_types = ('ScheduleRulesetPhProperties',
                       'ScheduleRulesetPhPropertiesAbridged')
        if _dict['type'] not in valid_types:
            raise ScheduleRulesetPhProperties_FromDictError(valid_types, _dict['type'])

        new_prop = cls(host)
        new_prop.id_num = _dict['id_num']
        new_prop.operating_weeks_year = _dict['operating_weeks_year']
        new_prop.operating_days_wk = _dict['operating_days_wk']
        new_prop.daily_operating_periods = DailyOperatingPeriodCollection.from_dict(
            _dict.get('operating_periods', {}))
        return new_prop

    def apply_properties_from_dict(self, abridged_data):
        return

    def ToString(self):
        """Overwrite .NET ToString."""
        return self.__repr__()

    def __repr__(self):
        """Properties representation."""
        return '{}'.format(self.__class__.__name__)



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Passive-House Style Monthly Climate Data"""

try:
    from itertools import izip as zip  # type: ignore
except ImportError:
    pass  # Python3

try:
    from typing import Dict, List, Collection, Any, Union, Optional
except ImportError:
    pass  # IronPython 2.7

from honeybee_ph import _base


class Climate_MonthlyValueSet(_base._Base):
    """A set of 12 monthly values (temp, radiation, etc)."""

    months = [
        "january",
        "february",
        "march",
        "april",
        "may",
        "june",
        "july",
        "august",
        "september",
        "october",
        "november",
        "december",
    ]

    def __init__(self, _values=[0.0]*12):
        # type: (Collection[float]) -> None
        super(Climate_MonthlyValueSet, self).__init__()
        self.values = _values

    @property
    def values(self):
        # type: () -> List[float]
        return [getattr(self, month) for month in self.months]

    @values.setter
    def values(self, _in):
        # type: (Collection[float]) -> None
        if (_in is None) or (len(_in) != 12):
            msg = "Error: Monthly data should be a collection of 12 numeric items.\n"\
                  "Got a {} of length: {}?".format(type(_in), len(_in))
            raise Exception(msg)

        for val, month_name in zip(_in, self.months):
            setattr(self, month_name, val)

    def to_dict(self):
        # type: () -> Dict[str, float]
        d = {}

        for month in self.months:
            d[month] = getattr(self, month)

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, float]) -> Climate_MonthlyValueSet
        obj = cls()

        for month in cls.months:
            setattr(obj, month, _input_dict.get(month))

        return obj


class Climate_MonthlyTempCollection(_base._Base):
    """Collection class to organize monthly temperature values"""

    def __init__(self,
                 _air=Climate_MonthlyValueSet(),
                 _dewpoint=Climate_MonthlyValueSet(),
                 _sky=Climate_MonthlyValueSet(),
                 _ground=Climate_MonthlyValueSet()
                 ):
        # type: (Climate_MonthlyValueSet, Climate_MonthlyValueSet, Climate_MonthlyValueSet, Climate_MonthlyValueSet) -> None
        super(Climate_MonthlyTempCollection, self).__init__()
        self.air_temps = _air
        self.dewpoints = _dewpoint
        self.sky_temps = _sky
        self.ground_temps = _ground

    def to_dict(self):
        # type: () -> Dict[str, Dict[str, float]]
        d = {}

        d["identifier"] = self.identifier
        d["display_name"] = self.display_name
        d["air_temps"] = self.air_temps.to_dict()
        d["dewpoints"] = self.dewpoints.to_dict()
        d["sky_temps"] = self.sky_temps.to_dict()
        d["ground_temps"] = self.ground_temps.to_dict()

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, Dict[str, float]]) -> Climate_MonthlyTempCollection
        new_obj = cls(
            _air=Climate_MonthlyValueSet.from_dict(_input_dict["air_temps"]),
            _dewpoint=Climate_MonthlyValueSet.from_dict(_input_dict["dewpoints"]),
            _sky=Climate_MonthlyValueSet.from_dict(_input_dict["sky_temps"]),
            _ground=Climate_MonthlyValueSet.from_dict(_input_dict["ground_temps"]),
        )
        new_obj.identifier = _input_dict["identifier"]
        new_obj.display_name = _input_dict["display_name"]

        return new_obj


class Climate_MonthlyRadiationCollection(_base._Base):

    def __init__(self,
                 _north=Climate_MonthlyValueSet(),
                 _east=Climate_MonthlyValueSet(),
                 _south=Climate_MonthlyValueSet(),
                 _west=Climate_MonthlyValueSet(),
                 _glob=Climate_MonthlyValueSet()):
        super(Climate_MonthlyRadiationCollection, self).__init__()
        self.north = _north
        self.east = _east
        self.south = _south
        self.west = _west
        self.glob = _glob

    def to_dict(self):
        # type: () -> Dict[str, Dict[str, float]]
        d = {}

        d["identifier"] = self.identifier
        d["display_name"] = self.display_name
        d["north"] = self.north.to_dict()
        d["east"] = self.east.to_dict()
        d["south"] = self.south.to_dict()
        d["west"] = self.west.to_dict()
        d["glob"] = self.glob.to_dict()

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, Dict[str, float]]) -> Climate_MonthlyRadiationCollection
        new_obj = cls(
            _north=Climate_MonthlyValueSet.from_dict(_input_dict["north"]),
            _east=Climate_MonthlyValueSet.from_dict(_input_dict["east"]),
            _south=Climate_MonthlyValueSet.from_dict(_input_dict["south"]),
            _west=Climate_MonthlyValueSet.from_dict(_input_dict["west"]),
            _glob=Climate_MonthlyValueSet.from_dict(_input_dict["glob"]),
        )
        new_obj.identifier = _input_dict["identifier"]
        new_obj.display_name = _input_dict["display_name"]

        return new_obj


class Climate_PeakLoadValueSet(_base._Base):
    """A set of Peak Load data."""

    def __init__(self,
                 _temp=0.0,
                 _rad_north=0.0,
                 _rad_east=0.0,
                 _rad_south=0.0,
                 _rad_west=0.0,
                 _rad_global=0.0,
                 _dewpoint_temp=None,
                 _sky_temp=None,
                 _ground_temp=None,
                 ):
        # type: (float, float, float, float, float, float, Optional[float], Optional[float], Optional[float]) -> None
        super(Climate_PeakLoadValueSet, self).__init__()
        self.temp = _temp
        self.rad_north = _rad_north
        self.rad_east = _rad_east
        self.rad_south = _rad_south
        self.rad_west = _rad_west
        self.rad_global = _rad_global
        self.dewpoint = _dewpoint_temp
        self.sky_temp = _sky_temp
        self.ground_temp = _ground_temp

    def to_dict(self):
        # type: () -> Dict[str, float]
        d = {}

        d['temp'] = self.temp
        d['rad_north'] = self.rad_north
        d['rad_east'] = self.rad_east
        d['rad_south'] = self.rad_south
        d['rad_west'] = self.rad_west
        d['rad_global'] = self.rad_global
        d["dewpoint"] = self.dewpoint
        d["sky_temp"] = self.sky_temp
        d["ground_temp"] = self.ground_temp
        d["display_name"] = self.display_name
        d["identifier"] = self.identifier

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, float]) -> Climate_PeakLoadValueSet
        new_obj = cls()

        new_obj.temp = _input_dict["temp"]
        new_obj.rad_north = _input_dict["rad_north"]
        new_obj.rad_east = _input_dict["rad_east"]
        new_obj.rad_south = _input_dict["rad_south"]
        new_obj.rad_west = _input_dict["rad_west"]
        new_obj.rad_global = _input_dict["rad_global"]
        new_obj.dewpoint = _input_dict["dewpoint"]
        new_obj.sky_temp = _input_dict["sky_temp"]
        new_obj.ground_temp = _input_dict["ground_temp"]
        new_obj.display_name = _input_dict["display_name"]
        new_obj.identifier = _input_dict["identifier"]

        return new_obj


class Climate_PeakLoadCollection(_base._Base):
    """A Collection of Peak Loads (Heating and Cooling)."""

    def __init__(self,
                 _heat_load_1=Climate_PeakLoadValueSet(),
                 _heat_load_2=Climate_PeakLoadValueSet(),
                 _cooling_load_1=Climate_PeakLoadValueSet(),
                 _cooling_load_2=Climate_PeakLoadValueSet(),
                 ):
        # type: (Climate_PeakLoadValueSet, Climate_PeakLoadValueSet, Climate_PeakLoadValueSet, Climate_PeakLoadValueSet) -> None
        super(Climate_PeakLoadCollection, self).__init__()
        self.heat_load_1 = _heat_load_1
        self.heat_load_2 = _heat_load_2
        self.cooling_load_1 = _cooling_load_1
        self.cooling_load_2 = _cooling_load_2

    def to_dict(self):
        # type: () -> Dict[str, Union[Dict[str, float], str]]
        d = {}

        d["heat_load_1"] = self.heat_load_1.to_dict()
        d["heat_load_2"] = self.heat_load_2.to_dict()
        d["cooling_load_1"] = self.cooling_load_1.to_dict()
        d["cooling_load_2"] = self.cooling_load_2.to_dict()
        d["display_name"] = self.display_name
        d["identifier"] = self.identifier

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict) -> Climate_PeakLoadCollection
        new_obj = cls(
            Climate_PeakLoadValueSet.from_dict(_input_dict["heat_load_1"]),
            Climate_PeakLoadValueSet.from_dict(_input_dict["heat_load_2"]),
            Climate_PeakLoadValueSet.from_dict(_input_dict["cooling_load_1"]),
            Climate_PeakLoadValueSet.from_dict(_input_dict["cooling_load_2"]),
        )
        new_obj.display_name = _input_dict["display_name"]
        new_obj.identifier = _input_dict["identifier"]

        return new_obj


class Climate_Ground(_base._Base):
    def __init__(self):
        super(Climate_Ground, self).__init__()
        self.ground_thermal_conductivity = 2
        self.ground_heat_capacity = 1000
        self.ground_density = 2000
        self.depth_groundwater = 3
        self.flow_rate_groundwater = 0.05

    def to_dict(self):
        # type: () -> Dict
        d = {}

        d["ground_thermal_conductivity"] = self.ground_thermal_conductivity
        d["ground_heat_capacity"] = self.ground_heat_capacity
        d["ground_density"] = self.ground_density
        d["depth_groundwater"] = self.depth_groundwater
        d["flow_rate_groundwater"] = self.flow_rate_groundwater

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict) -> Climate_Ground
        new_obj = cls()

        new_obj.ground_thermal_conductivity = _input_dict.get(
            "ground_thermal_conductivity")
        new_obj.ground_heat_capacity = _input_dict.get(
            "ground_heat_capacity")
        new_obj.ground_density = _input_dict.get("ground_density")
        new_obj.depth_groundwater = _input_dict.get("depth_groundwater")
        new_obj.flow_rate_groundwater = _input_dict.get(
            "flow_rate_groundwater")

        return new_obj


class Climate(_base._Base):
    def __init__(self,
                 _display_name="New York",
                 _station_elevation=0.0,
                 _daily_temp_swing=8.0,
                 _average_wind_speed=4.0,
                 _monthly_temps=Climate_MonthlyTempCollection(),
                 _monthly_radiation=Climate_MonthlyRadiationCollection(),
                 _peak_loads=Climate_PeakLoadCollection()
                 ):
        # type: (str, float, float, float, Climate_MonthlyTempCollection, Climate_MonthlyRadiationCollection, Climate_PeakLoadCollection) -> None
        super(Climate, self).__init__()
        self.display_name = _display_name
        self.station_elevation = _station_elevation  # m
        self.summer_daily_temperature_swing = _daily_temp_swing  # Deg-K
        self.average_wind_speed = _average_wind_speed  # m/s

        self.ground = Climate_Ground()
        self.monthly_temps = _monthly_temps
        self.monthly_radiation = _monthly_radiation
        self.peak_loads = _peak_loads

    def to_dict(self):
        # type: () -> Dict
        d = {}

        d["display_name"] = self.display_name
        d["identifier"] = self.identifier
        d["station_elevation"] = self.station_elevation
        d["summer_daily_temperature_swing"] = self.summer_daily_temperature_swing
        d["average_wind_speed"] = self.average_wind_speed

        d["ground"] = self.ground.to_dict()

        d["monthly_temps"] = self.monthly_temps.to_dict()
        d["monthly_radiation"] = self.monthly_radiation.to_dict()
        d["peak_loads"] = self.peak_loads.to_dict()

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict) -> Climate
        new_obj = cls()

        new_obj.display_name = _input_dict["display_name"]
        new_obj.identifier = _input_dict["identifier"]
        new_obj.station_elevation = _input_dict["station_elevation"]
        new_obj.summer_daily_temperature_swing = _input_dict["summer_daily_temperature_swing"]
        new_obj.average_wind_speed = _input_dict["average_wind_speed"]

        new_obj.ground = Climate_Ground.from_dict(
            _input_dict.get("ground", {}))
        new_obj.monthly_temps = Climate_MonthlyTempCollection.from_dict(
            _input_dict.get("monthly_temps", {}))
        new_obj.monthly_radiation = Climate_MonthlyRadiationCollection.from_dict(
            _input_dict.get("monthly_radiation", {}))
        new_obj.peak_loads = Climate_PeakLoadCollection.from_dict(
            _input_dict.get("peak_loads", {}))

        return new_obj

    def __copy__(self):
        # type: () -> Climate
        obj = Climate()
        obj.set_base_attrs_from_source(self)
        for attr_nm, attr_val in vars(self).items():
            setattr(obj, attr_nm, attr_val)
        return obj

    def duplicate(self):
        # type: () -> Climate
        return self.__copy__()


class Location(_base._Base):
    """Geographic Location Information."""

    def __init__(self,
                 latitude=40.6,
                 longitude=-73.8,
                 site_elevation=None,
                 climate_zone=1,
                 hours_from_UTC=-4,
                 ):
        # type: (float, float, Optional[float], int, int) -> None
        super(Location, self).__init__()
        self.latitude = latitude
        self.longitude = longitude
        self.site_elevation = site_elevation
        self.climate_zone = climate_zone
        self.hours_from_UTC = hours_from_UTC

    def to_dict(self):
        # type: () -> Dict
        d = {}

        d["latitude"] = self.latitude
        d["longitude"] = self.longitude
        d["site_elevation"] = self.site_elevation
        d["climate_zone"] = self.climate_zone
        d["hours_from_UTC"] = self.hours_from_UTC
        d["display_name"] = self.display_name
        d["identifier"] = self.identifier

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> Location
        new_obj = cls(
            _input_dict["latitude"],
            _input_dict["longitude"],
            _input_dict["site_elevation"],
            _input_dict["climate_zone"],
            _input_dict["hours_from_UTC"],
        )
        new_obj.display_name = _input_dict["display_name"]
        new_obj.identifier = _input_dict["identifier"]

        return new_obj

    def __copy__(self):
        # type: () -> Location
        obj = Location(
            self.latitude,
            self.longitude,
            self.site_elevation,
            self.climate_zone,
            self.hours_from_UTC,
        )
        obj.set_base_attrs_from_source(self)

        return obj

    def duplicate(self):
        # type: () -> Location
        return self.__copy__()


class PHPPCodes(_base._Base):
    """Settings / names if using Pre-loaded PHPP Library Data"""

    def __init__(self,
                 _country_code="US-United States of America",
                 _region_code="New York",
                 _dataset_name="US0055b-New York",
                 ):
        # type: (str, str, str) -> None
        super(PHPPCodes, self).__init__()
        self.country_code = _country_code
        self.region_code = _region_code
        self.dataset_name = _dataset_name

    def to_dict(self):
        # type: () -> Dict
        d = {}

        d["country_code"] = self.country_code
        d["region_code"] = self.region_code
        d["dataset_name"] = self.dataset_name
        d["display_name"] = self.dataset_name
        d["identifier"] = self.identifier

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict) -> PHPPCodes
        obj = cls(
            _input_dict["country_code"],
            _input_dict["region_code"],
            _input_dict["dataset_name"],
        )
        obj.display_name = _input_dict["display_name"]
        obj.identifier = _input_dict["identifier"]
        return obj

    def __copy__(self):
        # type: () -> PHPPCodes
        obj = PHPPCodes(
            self.country_code,
            self.region_code,
            self.dataset_name,
        )
        obj.set_base_attrs_from_source(self)
        return obj

    def duplicate(self):
        # type: () -> PHPPCodes
        return self.__copy__()


class Site(_base._Base):
    """Location and Climate data for the building site."""

    def __init__(self,
                 _location=Location(),
                 _climate=Climate(),
                 _phpp_library_codes=PHPPCodes(),
                 ):
        super(Site, self).__init__()
        self.location = _location
        self.climate = _climate
        self.phpp_library_codes = _phpp_library_codes

    def to_dict(self):
        # type: () -> Dict[str, Any]
        d = {}

        d['location'] = self.location.to_dict()
        d['climate'] = self.climate.to_dict()
        d['phpp_library_codes'] = self.phpp_library_codes.to_dict()
        d['display_name'] = self.display_name
        d['identifier'] = self.identifier

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, Any]) -> Site
        obj = cls(
            Location.from_dict(_input_dict['location']),
            Climate.from_dict(_input_dict['climate']),
            PHPPCodes.from_dict(_input_dict['phpp_library_codes']),
        )
        obj.display_name = _input_dict["display_name"]
        obj.identifier = _input_dict["identifier"]

        return obj

    def __copy__(self):
        # type: () -> Site
        obj = Site()

        obj.set_base_attrs_from_source(self)
        obj.location = self.location.duplicate()
        obj.climate = self.climate.duplicate()
        obj.phpp_library_codes = self.phpp_library_codes.duplicate()

        return obj

    def duplicate(self):
        # type: () -> Site
        return self.__copy__()



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Electric Equipment"""

try:
    from typing import Any
except:
    pass  # IronPython

from honeybee_energy_ph.load import ph_equipment


class ElectricEquipmentPhProperties_FromDictError(Exception):
    def __init__(self, _expected_types, _input_type):
        self.msg = 'Error: Expected type of "{}". Got: {}'.format(
            _expected_types, _input_type)
        super(ElectricEquipmentPhProperties_FromDictError, self).__init__(self.msg)


class ElectricEquipmentPhProperties(object):
    def __init__(self, _host):
        self._host = _host
        self.equipment_collection = ph_equipment.PhEquipmentCollection(self)

    @property
    def host(self):
        return self._host

    def to_dict(self, abridged=False):
        # type: (bool) -> dict
        d = {}

        if abridged:
            d['type'] = 'ElectricEquipmentPhPropertiesAbridged'
        else:
            d['type'] = 'ElectricEquipmentPhProperties'

        d['equipment_collection'] = self.equipment_collection.to_dict()

        return {'ph': d}

    @classmethod
    def from_dict(cls, _input_dict, _host):
        # type: (dict, Any) -> ElectricEquipmentPhProperties
        valid_types = ('ElectricEquipmentPhProperties',
                       'ElectricEquipmentPhPropertiesAbridged')
        if _input_dict['type'] not in valid_types:
            raise ElectricEquipmentPhProperties_FromDictError(
                valid_types, _input_dict['type'])

        new_prop = cls(_host)

        new_prop.equipment_collection = ph_equipment.PhEquipmentCollection.from_dict(
            _input_dict['equipment_collection'], _host=new_prop)

        return new_prop

    def apply_properties_from_dict(self, abridged_data):
        return

    def __copy__(self, new_host=None):
        # type: (Any) -> ElectricEquipmentPhProperties
        host = new_host or self._host
        new_obj = self.__class__(host)
        new_obj.equipment_collection = self.equipment_collection.duplicate(host)
        return new_obj

    def duplicate(self, new_host=None):
        # type: (Any) -> ElectricEquipmentPhProperties
        return self.__copy__(new_host)

    def __str__(self):
        return '{}(equipment_collection={})'.format(self.__class__.__name__, self.equipment_collection)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""PH-Properties classes for SHWSystem (System / Equipment) objects."""

try:
    from typing import Any, ValuesView, Dict
except:
    pass  # IronPython

from honeybee_energy_ph.hvac import hot_water


class SHWSystemPhProperties_FromDictError(Exception):
    def __init__(self, _expected_types, _input_type):
        self.msg = 'Error: Expected type of "{}". Got: {}'.format(
            _expected_types, _input_type)
        super(SHWSystemPhProperties_FromDictError, self).__init__(self.msg)


class SHWSystemPhProperties(object):
    """Honeybee-PH Properties for logging PH-style data."""

    def __init__(self, _host):
        self._host = _host
        self.id_num = 0

        self.tank_1 = None  # Optional[PhSHWTank]
        self.tank_2 = None  # Optional[PhSHWTank]
        self.tank_buffer = None  # Optional[PhSHWTank]
        self.tank_solar = None  # Optional[PhSHWTank]

        self._heaters = {}  # type: Dict[str, hot_water.PhHotWaterHeater]
        self._branch_piping = {}  # type: Dict[str, hot_water.PhPipeElement]
        self._recirc_piping = {}  # type: Dict[str, hot_water.PhPipeElement]

    @property
    def heaters(self):
        # type: () -> ValuesView[hot_water.PhHotWaterHeater]
        """Returns a list of all the heaters on the system."""
        return self._heaters.values()

    def clear_heaters(self):
        self._heaters = {}

    def add_heater(self, _h):
        # type: (hot_water.PhHotWaterHeater) -> None
        """Adds a new hot-water heater to the system."""
        if not _h:
            return

        assert hasattr(
            _h, 'to_dict'), 'Error: HW-Heater "{}" is not serializable?'.format(_h)
        self._heaters[_h.identifier] = _h

    def add_branch_piping(self, _branch_piping):
        # type: (hot_water.PhPipeElement) -> None
        self._branch_piping[_branch_piping.identifier] = _branch_piping

    def clear_branch_piping(self):
        self._branch_piping = {}

    @property
    def branch_piping(self):
        # type: () -> ValuesView[hot_water.PhPipeElement]
        """Returns a list of all the branch-piping objects in the system."""
        return self._branch_piping.values()

    def add_recirc_piping(self, _recirc_piping):
        # type: (hot_water.PhPipeElement) -> None
        self._recirc_piping[_recirc_piping.identifier] = _recirc_piping

    def clear_recirc_piping(self):
        self._recirc_piping = {}

    @property
    def recirc_piping(self):
        # type: () -> ValuesView[hot_water.PhPipeElement]
        """Returns a list of all the recirculation-piping objects in the system."""
        return self._recirc_piping.values()

    @property
    def host(self):
        return self._host

    @property
    def tanks(self):
        # type: () -> list[hot_water.PhSHWTank | None]
        """Return a list of the system tanks in order (1, 2, buffer, solar)."""
        return [self.tank_1, self.tank_2,  self.tank_buffer, self.tank_solar]

    def to_dict(self, abridged=False):
        # type: (bool) -> dict[str, dict]
        d = {}
        if abridged:
            d['type'] = 'SHWSystemPhPropertiesAbridged'
        else:
            d['type'] = 'SHWSystemPhProperties'

        d['id_num'] = self.id_num
        if self.tank_1:
            d['tank_1'] = self.tank_1.to_dict()
        if self.tank_2:
            d['tank_2'] = self.tank_2.to_dict()
        if self.tank_buffer:
            d['tank_buffer'] = self.tank_buffer.to_dict()
        if self.tank_solar:
            d['tank_solar'] = self.tank_solar.to_dict()

        d['heaters'] = {}
        for heater in self.heaters:
            d['heaters'][id(heater)] = heater.to_dict()

        d['branch_piping'] = {}
        for branch_piping in self.branch_piping:
            d['branch_piping'][branch_piping.identifier] = branch_piping.to_dict()

        d['recirc_piping'] = {}
        for recirc_piping in self.recirc_piping:
            d['recirc_piping'][recirc_piping.identifier] = recirc_piping.to_dict()

        return {'ph': d}

    @classmethod
    def from_dict(cls, _input_dict, _host):
        # type: (dict, Any) -> SHWSystemPhProperties
        valid_types = ('SHWSystemPhProperties',
                       'SHWSystemPhPropertiesAbridged')
        if _input_dict['type'] not in valid_types:
            raise SHWSystemPhProperties_FromDictError(valid_types, _input_dict['type'])

        new_prop = cls(_host)
        new_prop.id_num = _input_dict['id_num']

        if _input_dict.get('tank_1', None):
            new_prop.tank_1 = hot_water.PhSHWTank.from_dict(
                _input_dict['tank_1'])
        if _input_dict.get('tank_2', None):
            new_prop.tank_2 = hot_water.PhSHWTank.from_dict(
                _input_dict['tank_2'])
        if _input_dict.get('tank_buffer', None):
            new_prop.tank_buffer = hot_water.PhSHWTank.from_dict(
                _input_dict['tank_buffer'])
        if _input_dict.get('tank_buffer', None):
            new_prop.tank_buffer = hot_water.PhSHWTank.from_dict(
                _input_dict['tank_buffer'])

        for heater_dict in _input_dict['heaters'].values():
            new_prop.add_heater(hot_water.PhSHWHeaterBuilder.from_dict(heater_dict))

        for branch_piping_dict in _input_dict['branch_piping'].values():
            new_prop.add_branch_piping(
                hot_water.PhPipeElement.from_dict(branch_piping_dict))

        for recirc_piping_dict in _input_dict['recirc_piping'].values():
            new_prop.add_recirc_piping(
                hot_water.PhPipeElement.from_dict(recirc_piping_dict))

        return new_prop

    def apply_properties_from_dict(self, abridged_data):
        return

    def __copy__(self, new_host=None):
        # type: (Any) -> SHWSystemPhProperties
        _host = new_host or self._host
        new_obj = SHWSystemPhProperties(_host)
        new_obj.id_num = self.id_num

        if self.tank_1:
            new_obj.tank_1 = self.tank_1.duplicate()
        if self.tank_2:
            new_obj.tank_2 = self.tank_2.duplicate()
        if self.tank_buffer:
            new_obj.tank_buffer = self.tank_buffer.duplicate()
        if self.tank_solar:
            new_obj.tank_solar = self.tank_solar.duplicate()

        for k, v in self._heaters.items():
            new_obj._heaters[k] = v

        for k, v in self._branch_piping.items():
            new_obj._branch_piping[k] = v.duplicate()

        for k, v in self._recirc_piping.items():
            new_obj._recirc_piping[k] = v.duplicate()

        return new_obj

    def duplicate(self, new_host=None):
        # type: (Any) -> SHWSystemPhProperties
        return self.__copy__(new_host)

    def __str__(self):
        return '{}: id={}'.format(self.__class__.__name__, self.id_num)

    def __repr__(self):
        """Properties representation."""
        return '{!r}(id_num={!r}, tank_1={!r}, tank_2={!r}, tank_buffer={!r}, tank_solar={!r})'.format(
            self.__class__.__name__, self.id_num, self.tank_1, self.tank_2, self.tank_buffer, self.tank_solar)

    def ToString(self):
        """Overwrite .NET ToString."""
        return self.__repr__()

    def __add__(self, other):
        # type: (SHWSystemPhProperties) -> SHWSystemPhProperties
        new_obj = self.duplicate()

        for heater in self.heaters:
            new_obj.add_heater(heater)
        for heater in other.heaters:
            new_obj.add_heater(heater)

        for branch_pipe in self.branch_piping:
            new_obj.add_branch_piping(branch_pipe)
        for branch_pipe in other.branch_piping:
            new_obj.add_branch_piping(branch_pipe)

        for recirc_pipe in self.recirc_piping:
            new_obj.add_recirc_piping(recirc_pipe)
        for recirc_pipe in other.recirc_piping:
            new_obj.add_recirc_piping(recirc_pipe)

        return new_obj

    def __radd__(self, other):
        # type: (SHWSystemPhProperties) -> SHWSystemPhProperties
        if isinstance(other, int):
            return self
        else:
            return self + other



# -*- Python Version: 2.7 -*-
# -*- coding: utf-8 -*-

"""HBPH Base class for all Constructions"""

try:
    from typing import Optional, Any
except ImportError:
    pass  # Python 2.7

from honeybee.typing import valid_ep_string


class _Base(object):

    def __init__(self, _identifier):
        self.id_num = 0
        self._identifier = _identifier  # type: str
        self._display_name = None  # type: Optional[str]
        self._user_data = {}  # type: dict

    @property
    def identifier(self):
        """Get the text string for unique construction identifier."""
        return self._identifier

    @identifier.setter
    def identifier(self, identifier):
        self._identifier = valid_ep_string(identifier, 'construction identifier')

    @property
    def display_name(self):
        """Get or set a string for the object name without any character restrictions.

        If not set, this will be equal to the identifier.
        """
        if self._display_name is None:
            return self._identifier
        return self._display_name

    @display_name.setter
    def display_name(self, value):
        try:
            self._display_name = str(value)
        except UnicodeEncodeError:  # Python 2 machine lacking the character set
            self._display_name = value  # keep it as unicode

    @property
    def user_data(self):
        # type: () -> dict
        """Get an optional dictionary for additional meta data for this object.

        This will be None until it has been set. All keys and values of this
        dictionary should be of a standard Python type to ensure correct
        serialization of the object to/from JSON (eg. str, float, int, list, dict)
        """
        return self._user_data

    @user_data.setter
    def user_data(self, value):
        # type: (dict) -> None
        if value is not None:
            assert isinstance(value, dict), 'Expected dictionary for honeybee_energy_ph' \
                'object user_data. Got {}.'.format(type(value))
        self._user_data = value

    def to_dict(self):
        # type: () -> dict
        """Return all the base attribute values as a dict."""
        d = {}
        d['id_num'] = self.id_num
        d['identifier'] = str(self.identifier)
        d['display_name'] = self.display_name
        d['user_data'] = self.user_data
        return d

    def set_base_attrs_from_dict(self, _input_dict):
        # type: (dict[str, Any]) -> None
        """Set all the Base attribute values from an input dict."""
        self.id_num = _input_dict['id_num']
        self.identifier = _input_dict['identifier']
        self.display_name = _input_dict['display_name']
        self.user_data = _input_dict['user_data']
        return None

    def set_base_attrs_from_obj(self, other):
        # type: (_Base) -> None
        """Sets the base attributes based on another object. Used during __copy__."""
        self.id_num = other.id_num
        self.identifier = other.identifier
        self.display_name = other.display_name
        self.user_data = other.user_data.copy()
        return None

    def __str__(self):
        return '{}()'.format(self.__class__.__name__)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""HBPH Heating Objects"""

import sys
try:
    from typing import Any, Optional, Sequence
except ImportError:
    pass  # IronPython 2.7

from honeybee_energy_ph.hvac import _base, fuels


class UnknownPhHeatingTypeError(Exception):
    def __init__(self, _heater_types, _received_type):
        # type: (list[str], str) -> None
        self.msg = 'Error: Unknown HBPH-Heating-SubSystem type? Got: "{}" but only types: {} are allowed?'.format(
            _received_type, _heater_types)
        super(UnknownPhHeatingTypeError, self).__init__(self.msg)


class PhHeatingSystem(_base._PhHVACBase):
    """Base class for all PH-Heating Systems (elec, boiler, etc...)"""

    def __init__(self):
        super(PhHeatingSystem, self).__init__()
        self.heating_type = self.__class__.__name__
        self.percent_coverage = 1.0

    def to_dict(self):
        # type: () -> dict
        d = {}
        d['identifier'] = self.identifier
        d['display_name'] = self.display_name
        d['heating_type'] = self.heating_type
        d['percent_coverage'] = self.percent_coverage
        return d

    def base_attrs_from_dict(self, _input_dict):
        # type: (PhHeatingSystem, dict) -> PhHeatingSystem
        self.identifier = _input_dict['identifier']
        self.display_name = _input_dict['display_name']
        self.heating_type = _input_dict['heating_type']
        self.percent_coverage = _input_dict['percent_coverage']
        return self

    def check_dict_type(self, _input_dict):
        # type: (dict) -> None
        """Check that the input dict type is correct for the Heating System being constructed."""
        heating_type = _input_dict['heating_type']
        msg = "Error creating Heating System from dict. Expected '{}' but got '{}'".format(
            self.__class__.__name__, heating_type)
        assert heating_type == str(self.__class__.__name__), msg
        return None


# -----------------------------------------------------------------------------
# Heating Types


class PhHeatingDirectElectric(PhHeatingSystem):
    """Heating via direct-electric (resistance heating)."""

    def __init__(self):
        super(PhHeatingDirectElectric, self).__init__()

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = super(PhHeatingDirectElectric, self).to_dict()
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhHeatingDirectElectric
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)
        return new_obj


class PhHeatingFossilBoiler(PhHeatingSystem):
    """Heating via boiler using fossil-fuel (gas, oil)"""

    def __init__(self):
        super(PhHeatingFossilBoiler, self).__init__()
        self.fuel = fuels.NATURAL_GAS
        self.condensing = True
        self.in_conditioned_space = True
        self.effic_at_30_percent_load = 0.98
        self.effic_at_nominal_load = 0.94
        self.avg_rtrn_temp_at_30_percent_load = 30
        self.avg_temp_at_70C_55C = 41
        self.avg_temp_at_55C_45C = 35
        self.avg_temp_at_32C_28C = 24

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = super(PhHeatingFossilBoiler, self).to_dict()

        d['fuel'] = self.fuel
        d['condensing'] = self.condensing
        d['in_conditioned_space'] = self.in_conditioned_space
        d['effic_at_30_percent_load'] = self.effic_at_30_percent_load
        d['effic_at_nominal_load'] = self.effic_at_nominal_load
        d['avg_rtrn_temp_at_30_percent_load'] = self.avg_rtrn_temp_at_30_percent_load
        d['avg_temp_at_70C_55C'] = self.avg_temp_at_70C_55C
        d['avg_temp_at_55C_45C'] = self.avg_temp_at_55C_45C
        d['avg_temp_at_32C_28C'] = self.avg_temp_at_32C_28C

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhHeatingFossilBoiler
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)

        new_obj.fuel = _input_dict['fuel']
        new_obj.condensing = _input_dict['condensing']
        new_obj.in_conditioned_space = _input_dict['in_conditioned_space']
        new_obj.effic_at_30_percent_load = _input_dict['effic_at_30_percent_load']
        new_obj.effic_at_nominal_load = _input_dict['effic_at_nominal_load']
        new_obj.avg_rtrn_temp_at_30_percent_load = _input_dict['avg_rtrn_temp_at_30_percent_load']
        new_obj.avg_temp_at_70C_55C = _input_dict['avg_temp_at_70C_55C']
        new_obj.avg_temp_at_55C_45C = _input_dict['avg_temp_at_55C_45C']
        new_obj.avg_temp_at_32C_28C = _input_dict['avg_temp_at_32C_28C']

        return new_obj


class PhHeatingWoodBoiler(PhHeatingSystem):
    """Heating via boiler using wood (log, pellet)."""

    def __init__(self):
        super(PhHeatingWoodBoiler, self).__init__()
        self.fuel = fuels.WOOD_LOG
        self.in_conditioned_space = True
        self.effic_in_basic_cycle = 0.6
        self.effic_in_const_operation = 0.7
        self.avg_frac_heat_output = 0.4
        self.temp_diff_on_off = 30
        self.rated_capacity = 15  # kW
        self.demand_basic_cycle = 1  # kWh
        self.power_stationary_run = 1  # W
        self.power_standard_run = None  # type: Optional[float]
        self.no_transport_pellets = None  # type: Optional[bool]
        self.only_control = None  # type: Optional[bool]
        self.area_mech_room = None  # type: Optional[float]

    @property
    def useful_heat_output(self):
        return 0.9 * self.rated_capacity  # kWH

    @property
    def avg_power_output(self):
        return 0.5 * self.rated_capacity  # kW

    def to_dict(self):
        # type: () -> dict
        d = super(PhHeatingWoodBoiler, self).to_dict()

        d['fuel'] = self.fuel
        d['in_conditioned_space'] = self.in_conditioned_space
        d['effic_in_basic_cycle'] = self.effic_in_basic_cycle
        d['effic_in_const_operation'] = self.effic_in_const_operation
        d['avg_frac_heat_output'] = self.avg_frac_heat_output
        d['temp_diff_on_off'] = self.temp_diff_on_off
        d['rated_capacity'] = self.rated_capacity
        d['demand_basic_cycle'] = self.demand_basic_cycle
        d['power_stationary_run'] = self.power_stationary_run
        d['power_standard_run'] = self.power_standard_run
        d['no_transport_pellets'] = self.no_transport_pellets
        d['only_control'] = self.only_control
        d['area_mech_room'] = self.area_mech_room

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhHeatingWoodBoiler

        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)

        new_obj.fuel = _input_dict['fuel']
        new_obj.in_conditioned_space = _input_dict['in_conditioned_space']
        new_obj.effic_in_basic_cycle = _input_dict['effic_in_basic_cycle']
        new_obj.effic_in_const_operation = _input_dict['effic_in_const_operation']
        new_obj.avg_frac_heat_output = _input_dict['avg_frac_heat_output']
        new_obj.temp_diff_on_off = _input_dict['temp_diff_on_off']
        new_obj.rated_capacity = _input_dict['rated_capacity']
        new_obj.demand_basic_cycle = _input_dict['demand_basic_cycle']
        new_obj.power_stationary_run = _input_dict['power_stationary_run']
        new_obj.no_transport_pellets = _input_dict['no_transport_pellets']
        new_obj.only_control = _input_dict['only_control']
        new_obj.area_mech_room = _input_dict['area_mech_room']

        return new_obj


class PhHeatingDistrict(PhHeatingSystem):
    """Heating via district-heat."""

    def __init__(self):
        super(PhHeatingDistrict, self).__init__()
        self.fuel = fuels.GAS_CGS_70_PHC
        self.util_factor_of_heat_transfer_station = 0.5

    def to_dict(self):
        # type: () -> dict
        d = super(PhHeatingDistrict, self).to_dict()
        d['fuel'] = self.fuel
        d['util_factor_of_heat_transfer_station'] = self.util_factor_of_heat_transfer_station
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhHeatingDistrict
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)

        new_obj.fuel = _input_dict['fuel']
        new_obj.util_factor_of_heat_transfer_station = _input_dict['util_factor_of_heat_transfer_station']
        return new_obj


class PhHeatingHeatPumpAnnual(PhHeatingSystem):

    def __init__(self):
        super(PhHeatingHeatPumpAnnual, self).__init__()
        self.annual_COP = 2.5
        self.total_system_perf_ratio = 0.4

    def to_dict(self):
        # type: () -> dict
        d = super(PhHeatingHeatPumpAnnual, self).to_dict()
        d['annual_COP'] = self.annual_COP
        d['total_system_perf_ratio'] = self.total_system_perf_ratio
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhHeatingHeatPumpAnnual
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)

        new_obj.annual_COP = _input_dict['annual_COP']
        new_obj.total_system_perf_ratio = _input_dict['total_system_perf_ratio']
        return new_obj


class PhHeatingHeatPumpRatedMonthly(PhHeatingSystem):
    """Heating via electric heat-pump."""

    def __init__(self):
        super(PhHeatingHeatPumpRatedMonthly, self).__init__()
        self.COP_1 = 2.5
        self.ambient_temp_1 = -8.333  # =17F
        self.COP_2 = 2.5
        self.ambient_temp_2 = 8.333  # =47F

    @property
    def monthly_COPS(self):
        return [self.COP_1, self.COP_2]

    @monthly_COPS.setter
    def monthly_COPS(self, _cops):
        # type: (Sequence[float]) -> None
        self.COP_1 = _cops[0]
        try:
            self.COP_2 = _cops[1]
        except IndexError:
            self.COP_2 = _cops[0]
        return

    @property
    def monthly_temps(self):
        return [self.ambient_temp_1, self.ambient_temp_2]

    @monthly_temps.setter
    def monthly_temps(self, _cops):
        # type: (Sequence[float]) -> None
        self.ambient_temp_1 = _cops[0]
        try:
            self.ambient_temp_2 = _cops[1]
        except IndexError:
            self.ambient_temp_2 = _cops[0]
        return

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = super(PhHeatingHeatPumpRatedMonthly, self).to_dict()
        d['COP_1'] = self.COP_1
        d['ambient_temp_1'] = self.ambient_temp_1
        d['COP_2'] = self.COP_2
        d['ambient_temp_2'] = self.ambient_temp_2
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhHeatingHeatPumpRatedMonthly
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)

        new_obj.COP_1 = _input_dict['COP_1']
        new_obj.ambient_temp_1 = _input_dict['ambient_temp_1']
        new_obj.COP_2 = _input_dict['COP_2']
        new_obj.ambient_temp_2 = _input_dict['ambient_temp_2']
        return new_obj


class PhHeatingHeatPumpCombined(PhHeatingSystem):

    def __init__(self):
        raise NotImplementedError()


# -----------------------------------------------------------------------------


class PhHeatingSystemBuilder(object):
    """Constructor class for PH-HeatingSystems"""

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhHeatingSystem
        """Find the right appliance constructor class from the module based on the 'type' name."""
        heating_type = _input_dict.get('heating_type')

        valid_class_types = [nm for nm in dir(
            sys.modules[__name__]) if nm.startswith('Ph')]

        if heating_type not in valid_class_types:
            raise UnknownPhHeatingTypeError(valid_class_types, heating_type)
        heating_class = getattr(sys.modules[__name__], heating_type)
        new_equipment = heating_class.from_dict(_input_dict)
        return new_equipment

    def __str__(self):
        return '{}()'.format(self.__class__.__name__)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Find Phius program data and build HBE-Programs."""

try:  # import the core honeybee dependencies
    from honeybee.typing import clean_and_id_ep_string, clean_ep_string
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:
    from honeybee_energy.load import lighting, people, equipment
    from honeybee_energy.schedule.ruleset import ScheduleRuleset
    from honeybee_energy.programtype import ProgramType
    from honeybee_energy.lib.scheduletypelimits import schedule_type_limit_by_identifier
    from honeybee_energy.lib.programtypes import program_type_by_identifier, \
        building_program_type_by_identifier
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_energy:\n\t{}'.format(e))

from re import L
from honeybee_ph_standards.programtypes import PHIUS_programs
from honeybee_energy_ph.properties import ruleset


class MissingBaseProgramError(Exception):
    def __init__(self, _data):
        self.msg = 'Error: The Phius data set for "{}" appears to missing '\
            'a Honeybee base-program?'.format(_data['name'])
        super(MissingBaseProgramError, self).__init__(self.msg)


def _clean_str(_str):
    # type: (str) -> str
    if not _str:
        return ''
    else:
        return str(_str).lstrip().rstrip().upper().replace(' ', '_').replace('-', '_')


def load_data_from_Phius_standards(_search_key, _search_field='name', _protocol=''):
    # type: (str, str, str) -> list[dict[str, dict]]
    """Returns a list of Phius program data as dicts.

    Arguments:
    ----------
        * _search_key (str): The key to search the Phius library for.
        * _search_field (str): default='name' | The dict field to search.
        # _protocol (str): Optional extra flag to filter by the specific 'protocol'

    Returns:
    --------
        * (list[dict]): A list of the Phius data found in the library.
    """
    if not _search_key:
        return []

    prog_data = []
    _search_key = _clean_str(_search_key)
    _protocol = _clean_str(_protocol)
    for data in PHIUS_programs.PHIUS_library.values():
        if _search_key not in _clean_str(data[_search_field]):
            continue
        if not _protocol:
            prog_data.append(data)
        else:
            if _protocol in _clean_str(data['protocol']):
                prog_data.append(data)

    return prog_data


def build_hb_people_from_Phius_data(_data):
    # type: (dict) -> people.People
    """Returns a new HBE-People Object with attributes based on the input data dict

    Arguments:
    ----------
        * _data (dict): The "people" element from a full Phius data dict.

    Returns:
    --------
        * (people.People): A new HB-People object.
    """
    # -------------------------------------------------------------------------
    # -- Build the People Schedule and set attributes
    ph_sched_ph_prop = ruleset.ScheduleRulesetPhProperties.from_days_per_year(
        _data['schedule']['annual_utilization_days'],
        _host=None
    )
    ph_sched_ph_prop.daily_operating_periods.add_period_to_collection(
        ruleset.DailyOperationPeriod.from_start_end_hours(
            _data['schedule']['start_hour'],
            _data['schedule']['end_hour'],
            _data['schedule']['relative_utilization_factor'],
            'operating'
        )
    )
    ppl_sched = ScheduleRuleset.from_constant_value(
        _data['schedule']['name'],
        ph_sched_ph_prop.annual_average_operating_fraction,
        schedule_type_limit_by_identifier('Fractional')
    )
    ph_sched_ph_prop._host = ppl_sched._properties
    ppl_sched._properties._ph = ph_sched_ph_prop

    # -------------------------------------------------------------------------
    # -- Build the HBE-People and set attributes
    hb_people = people.People(
        _data['loads']['identifier'],
        _data['loads']['people_per_area'],
        ppl_sched
    )

    return hb_people


def build_hb_lighting_from_Phius_data(_data):
    # type: (dict) -> lighting.Lighting
    """Returns a new HBE-Lighting Object with attributes based on the input data dict

    Arguments:
    ----------
        * _data (dict): The "lighting" element from a full Phius data dict.

    Returns:
    --------
        * (lighting.Lighting): A new HB-Lighting object.
    """
    # -------------------------------------------------------------------------
    # -- Build the Lighting Schedule and set attributes
    ph_sched_ph_prop = ruleset.ScheduleRulesetPhProperties.from_days_per_year(
        _data['schedule']['annual_utilization_days'],
        _host=None
    )
    ph_sched_ph_prop.daily_operating_periods.add_period_to_collection(
        ruleset.DailyOperationPeriod.from_operating_hours(
            _data['schedule']['daily_operating_hours'],
            _data['schedule']['relative_utilization_factor'],
            'operating'
        )
    )
    lighting_sched = ScheduleRuleset.from_constant_value(
        _data['schedule']['name'],
        ph_sched_ph_prop.annual_average_operating_fraction,
        schedule_type_limit_by_identifier('Fractional')
    )
    ph_sched_ph_prop._host = lighting_sched._properties
    lighting_sched._properties._ph = ph_sched_ph_prop

    # -------------------------------------------------------------------------
    # -- Build the HBE-Lighting and set attributes
    hb_lighting = lighting.Lighting(
        _data['loads']['name'],
        _data['loads']['watts_per_area'],
        lighting_sched,
        0.0,
        0.32,
        0.25
    )

    # -- Set LightingPhProperties attributes
    hb_lighting.properties.ph.target_lux = _data['loads']['target_lux']
    hb_lighting.properties.ph.target_lux_height = _data['loads']['target_lux_height']

    return hb_lighting


def build_hb_elec_equip_from_Phius_data(_data):
    # type: (dict) -> equipment.ElectricEquipment
    """Returns a new HBE-Equipment Object with attributes based on the input data dict

    Arguments:
    ----------
        * _data (dict): The "elec_equipment" element from a full Phius data dict.

    Returns:
    --------
        * (people.People): A new HB-People object.
    """
    # -------------------------------------------------------------------------
    # -- Build the Equipment Schedule and set attributes
    ph_sched_ph_prop = ruleset.ScheduleRulesetPhProperties(_host=None)
    ph_sched_ph_prop.daily_operating_periods.add_period_to_collection(
        ruleset.DailyOperationPeriod.from_annual_utilization_factor(
            _data['schedule']['annual_utilization_factor'],
            'operating'
        )
    )
    equip_sched = ScheduleRuleset.from_constant_value(
        _data['schedule']['name'],
        ph_sched_ph_prop.annual_average_operating_fraction,
        schedule_type_limit_by_identifier('Fractional')
    )
    ph_sched_ph_prop._host = equip_sched._properties
    equip_sched._properties._ph = ph_sched_ph_prop

    # -------------------------------------------------------------------------
    # -- Build the HBE-Equipment and set attributes
    hb_elec_equip = equipment.ElectricEquipment(
        _data['loads']['identifier'],
        _data['loads']['watts_per_area'],
        equip_sched
    )

    return hb_elec_equip


def build_hb_program_from_Phius_data(_data):
    # type: (dict) -> None
    """Return a new HB-Progam with attributes based on an input Phius dataset

    Arguments:
    ---------
        * _data (dict): The full Phius datadict

    Returns:
    --------
        * (program): The new Honeybee-Energy Program.
    """

    # -- Sort out the base-program
    base_program = _data['hb_base_program']
    if not base_program:
        raise MissingBaseProgramError(_data)
    try:
        base_program_ = building_program_type_by_identifier(base_program)
    except ValueError:
        base_program_ = program_type_by_identifier(base_program)
    program = base_program_.duplicate()
    program.identifier = clean_ep_string(
        "Phius_{}".format(_data['name'].replace(' ', '_')))
    program.display_name = "{}::{}".format(_data['protocol'], _data['name'])

    # -- Build and assign the sub-programs
    program.people = build_hb_people_from_Phius_data(_data['people'])
    program.lighting = build_hb_lighting_from_Phius_data(_data['lighting'])
    program.electric_equipment = build_hb_elec_equip_from_Phius_data(
        _data['elec_equipment'])

    # -- Inherit the rest from the base-program
    # program.gas_equipment = _gas_equip_
    # program.service_hot_water = _hot_water_
    # program.infiltration = _infiltration_
    # program.ventilation = _ventilation_
    # program.setpoint = _setpoint_

    return program



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""This is called during __init__ and extends the base honeybee class Properties with a new ._ph slot"""

from honeybee.properties import (
    ModelProperties,
    RoomProperties,
    FaceProperties,
    ApertureProperties,
    ShadeProperties
)

from .properties.space import SpaceProperties, SpacePhProperties
from .properties.model import ModelPhProperties
from .properties.room import RoomPhProperties
from .properties.face import FacePhProperties
from .properties.aperture import AperturePhProperties
from .properties.shade import ShadePhProperties

# Step 1)
# set a private ._ph attribute on each relevant HB-Core Property class to None

setattr(ModelProperties, '_ph', None)
setattr(RoomProperties, '_ph', None)
setattr(FaceProperties, '_ph', None)
setattr(ApertureProperties, '_ph', None)
setattr(SpaceProperties, '_ph', None)
setattr(ShadeProperties, '_ph', None)

# Step 2)
# create methods to define the public .ph property instances on each obj.properties container


def model_ph_properties(self):
    if self._ph is None:
        self._ph = ModelPhProperties(self.host)
    return self._ph


def room_ph_properties(self):
    if self._ph is None:
        self._ph = RoomPhProperties(self.host)
    return self._ph


def face_ph_properties(self):
    if self._ph is None:
        self._ph = FacePhProperties(self.host)
    return self._ph


def aperture_ph_properties(self):
    if self._ph is None:
        self._ph = AperturePhProperties(self.host)
    return self._ph


def space_ph_properties(self):
    if self._ph is None:
        self._ph = SpacePhProperties(self.host)
    return self._ph


def shade_ph_properties(self):
    if self._ph is None:
        self._ph = ShadePhProperties(self.host)
    return self._ph


# Step 3)
# add public .ph property methods to the Properties classes
setattr(ModelProperties, 'ph', property(model_ph_properties))
setattr(RoomProperties, 'ph', property(room_ph_properties))
setattr(FaceProperties, 'ph', property(face_ph_properties))
setattr(ApertureProperties, 'ph', property(aperture_ph_properties))
setattr(SpaceProperties, 'ph', property(space_ph_properties))
setattr(ShadeProperties, 'ph', property(shade_ph_properties))



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""HB-Shade Passive House (PH) Properties."""
try:
    from typing import Any
except ImportError:
    pass  # Python 2.7


class ShadePhProperties(object):

    def __init__(self, _host):
        self._host = _host
        self.id_num = 0

    @property
    def host(self):
        return self._host

    def duplicate(self, new_host=None):
        # type: (Any) -> ShadePhProperties
        _host = new_host or self._host
        new_properties_obj = ShadePhProperties(_host)
        new_properties_obj.id_num = self.id_num

        return new_properties_obj

    def ToString(self):
        return self.__repr__()

    def __repr__(self):
        return "Shade Passive House Properties: [host: {}]".format(self.host.display_name)

    def to_dict(self, abridged=False):
        # type: (bool) -> dict[str, dict]
        d = {}
        t = 'ShadePhProperties' if not \
            abridged else 'ShadePhPropertiesAbridged'
        d.update({'type': t})
        d.update({'id_num': self.id_num})

        return {'ph': d}

    @classmethod
    def from_dict(cls, data, host):
        # type: (dict, Any) -> ShadePhProperties
        assert data['type'] == 'ShadePhProperties', \
            'Expected ShadePhProperties. Got {}.'.format(data['type'])

        new_prop = cls(host)
        new_prop.id_num = data.get('id_num', 0)

        return new_prop



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setuptools.setup(
    name="honeybee-ph",
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    author="PH-Tools",
    author_email="phtools@bldgtyp.com",
    description="Plugin for Honeybee to enable Passive House modeling.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/PH-Tools/honeybee_ph",
    packages=setuptools.find_packages(
        exclude=[
            "tests",
            "diagrams",
            "docs",
            "honeybee_grasshopper_ph",
            "honeybee_ph_rhino",
        ]),
    include_package_data=True,
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
        "Typing :: Typed",
    ],
    license="GPLv3+"
)



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Properties for Honeybee-Energy-PH | Load | Lighting"""

try:
    from typing import Any
except ImportError:
    # Python 2.7
    pass


class LightingPhProperties_FromDictError(Exception):
    def __init__(self, _expected_types, _input_type):
        self.msg = 'Error: Expected type of "{}". Got: {}'.format(
            _expected_types, _input_type)
        super(LightingPhProperties_FromDictError, self).__init__(self.msg)


class LightingPhProperties(object):
    """Ph Properties for Honeybee-Energy Lighting"""

    def __init__(self, _host):
        self._host = _host
        self.id_num = 0
        self.target_lux = 300
        self.target_lux_height = 0.8  # m

    def duplicate(self, new_host=None):
        # type: (Any) -> LightingPhProperties
        _host = new_host or self._host

        new_prop_obj = self.__class__(_host)
        new_prop_obj.id_num = self.id_num
        new_prop_obj.target_lux = self.target_lux
        new_prop_obj.target_lux_height = self.target_lux_height

        return new_prop_obj

    @property
    def host(self):
        return self._host

    def __str__(self):
        return '{}()'.format(self.__class__.__name__)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)

    def to_dict(self, abridged=False):
        # type: (bool) -> dict[str, dict]
        d = {}

        if abridged:
            d['type'] = 'LightingPhPropertiesAbridged'
        else:
            d['type'] = 'LightingPhProperties'

        d['id_num'] = self.id_num
        d['target_lux'] = self.target_lux
        d['target_lux_height'] = self.target_lux_height

        return {'ph': d}

    @classmethod
    def from_dict(cls, data, host):
        # type: (dict, Any) -> LightingPhProperties
        valid_types = ('LightingPhProperties',
                       'LightingPhPropertiesAbridged')
        if data['type'] not in valid_types:
            raise LightingPhProperties_FromDictError(valid_types, data['type'])

        new_prop = cls(host)
        new_prop.id_num = data['id_num']
        new_prop.target_lux = data['target_lux']
        new_prop.target_lux_height = data['target_lux_height']

        return new_prop



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

try:
    from typing import Any, Dict, Optional
except ImportError:
    pass  # Python 2.7

from honeybee import properties


class SpaceProperties(properties._Properties):
    """Honeybee-PH Space Properties.

    Space properties. This class will be extended by extensions.

    Usage:

    .. code-block:: python

        space = Space()
        space.properties -> SpaceProperties
        space.properties.ph -> SpacePhProperties
        space.properties.energy -> SpaceEnergyProperties
    """

    def to_dict(self, abridged=False, include=None):
        """Convert properties to dictionary.

        Args:
            abridged: Boolean to note whether the full dictionary describing the
                object should be returned (False) or just an abridged version (True).
                Default: False.
            include: A list of keys to be included in dictionary.
                If None all the available keys will be included.
        """
        d = {}
        if abridged == False:
            d['type'] = 'SpaceProperties'
        else:
            d['type'] = 'SpacePropertiesAbridged'
        d = self._add_extension_attr_to_dict(d, abridged, include)
        return d

    @classmethod
    def from_dict(cls, _dict={}, _host=None):
        # type: (Dict, Any) -> SpaceProperties
        assert _dict['type'] == 'SpaceProperties', \
            'Expected SpaceProperties. Got {}.'.format(_dict['type'])

        obj = cls(_host)

        return obj

    def add_prefix(self, prefix):
        """Change the identifier extension attributes unique to this object by adding a prefix.

        Notably, this method only adds the prefix to extension attributes that must
        be unique to the Space (eg. single-room HVAC systems) and does not add the
        prefix to attributes that are shared across several Spaces (eg. ConstructionSets).

        Args:
            prefix: Text that will be inserted at the start of extension attribute identifiers.
        """
        self._add_prefix_extension_attr(prefix)

    def reset_to_default(self):
        """Reset the extension properties assigned at the level of this Room to default.

        This typically means erasing any ConstructionSets or ModifierSets assigned
        to this Space among other properties.
        """
        self._reset_extension_attr_to_default()

    def __repr__(self):
        """Properties representation."""
        return '{}: {}'.format(self.__class__.__name__, self.host.display_name)


class SpacePhProperties(object):

    def __init__(self, _host):
        self._host = _host
        self.id_num = 0

        # TODO: Temporary override until I figure out right right way
        self._v_sup = None
        self._v_eta = None
        self._v_tran = None

    @property
    def host(self):
        return self._host

    @property
    def has_ventilation_flow_rates(self):
        # type: () -> bool
        return any([self._v_sup, self._v_eta, self._v_tran])

    @property
    def honeybee_flow_rate(self):
        # type: () -> Optional[float]
        if not self.has_ventilation_flow_rates:
            return None
        return max([self._v_sup or 0, self._v_eta or 0, self._v_tran or 0])

    def duplicate(self, new_host=None):
        # type: (Any) -> SpacePhProperties
        _host = new_host or self._host
        new_properties_obj = SpacePhProperties(_host)

        new_properties_obj.id_num = self.id_num
        new_properties_obj._v_sup = self._v_sup
        new_properties_obj._v_eta = self._v_eta
        new_properties_obj._v_tran = self._v_tran

        return new_properties_obj

    def __str__(self):
        return "{}: [host: {}], v_eta={}, v_sup={}, v_tran={}".format(self.__class__.__name__, self.host.display_name, self._v_eta, self._v_sup, self._v_tran)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)

    def to_dict(self, abridged=False):
        # type: (bool) -> dict[str, dict]
        d = {}

        if abridged:
            d['type'] = 'SpacePhPropertiesAbridged'
        else:
            d['type'] = 'SpacePhProperties'

        d['id_num'] = self.id_num
        d['_v_sup'] = self._v_sup
        d['_v_eta'] = self._v_eta
        d['_v_tran'] = self._v_tran

        return {'ph': d}

    @classmethod
    def from_dict(cls, data, host):
        # type: (dict, Any) -> SpacePhProperties
        assert 'SpacePhProperties' in data['type'], \
            'Expected SpacePhProperties. Got {}.'.format(data['type'])
        new_prop = cls(host)

        new_prop.id_num = data['id_num']
        new_prop._v_sup = data['_v_sup']
        new_prop._v_eta = data['_v_eta']
        new_prop._v_tran = data['_v_tran']

        return new_prop



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Properties class for PH-HVAC IdealAir Systems"""

try:
    from typing import Any, Optional
except:
    pass  # IronPython

from honeybee_energy_ph.hvac import ventilation, heating, cooling


class IdealAirSystemPhProperties_FromDictError(Exception):
    def __init__(self, _expected_types, _input_type):
        self.msg = 'Error: Expected type of "{}". Got: {}'.format(
            _expected_types, _input_type)
        super(IdealAirSystemPhProperties_FromDictError, self).__init__(self.msg)


class IdealAirSystemPhProperties(object):
    """Honeybee-PH Properties for storing PH-style data."""

    def __init__(self, _host):
        self._host = _host
        self.id_num = 0
        self.ventilation_system = None  # type: Optional[ventilation.PhVentilationSystem]
        self.heating_systems = set()  # type: set[heating.PhHeatingSystem]
        self.cooling_systems = set()  # type: set[cooling.PhCoolingSystem]

    @property
    def host(self):
        return self._host

    def to_dict(self, abridged=False):
        # type: (bool) -> dict[str, dict]
        d = {}
        if abridged:
            d['type'] = 'IdealAirSystemPhPropertiesAbridged'
        else:
            d['type'] = 'IdealAirSystemPhProperties'

        d['id_num'] = self.id_num

        if self.ventilation_system:
            d['ventilation_system'] = self.ventilation_system.to_dict()
        else:
            d['ventilation_system'] = None

        d['heating_systems'] = []
        for system in self.heating_systems:
            if system is None:
                continue
            d['heating_systems'].append(system.to_dict())

        d['cooling_systems'] = []
        for system in self.cooling_systems:
            if system is None:
                continue
            d['cooling_systems'].append(system.to_dict())

        return {'ph': d}

    @classmethod
    def from_dict(cls, _input_dict, host):
        # type: (dict, Any) -> IdealAirSystemPhProperties
        valid_types = ('IdealAirSystemPhProperties',
                       'IdealAirSystemPhPropertiesAbridged')
        if _input_dict['type'] not in valid_types:
            raise IdealAirSystemPhProperties_FromDictError(
                valid_types, _input_dict['type'])

        new_prop = cls(host)
        new_prop.id_num = _input_dict['id_num']

        vent_sys_dict = _input_dict['ventilation_system']
        if vent_sys_dict:
            ph_vent_sys = ventilation.PhVentilationSystem.from_dict(vent_sys_dict)
            new_prop.ventilation_system = ph_vent_sys

        for htg_sys_dict in _input_dict.get('heating_systems', []):
            htg_sys = heating.PhHeatingSystemBuilder.from_dict(htg_sys_dict)
            new_prop.heating_systems.add(htg_sys)

        for cooling_sys_dict in _input_dict.get('cooling_systems', []):
            cooling_sys = cooling.PhCoolingSystemBuilder.from_dict(cooling_sys_dict)
            new_prop.cooling_systems.add(cooling_sys)

        return new_prop

    def apply_properties_from_dict(self, abridged_data):
        return

    def duplicate(self, new_host=None):
        # type: (Any) -> IdealAirSystemPhProperties
        return self.__copy__(new_host)

    def __copy__(self, new_host=None):
        # type: (Any) -> IdealAirSystemPhProperties
        _host = new_host or self._host
        new_obj = IdealAirSystemPhProperties(_host)

        new_obj.id_num = self.id_num
        new_obj.ventilation_system = self.ventilation_system
        for htg_sys in self.heating_systems:
            new_obj.heating_systems.add(htg_sys)
        for clg_sys in self.cooling_systems:
            new_obj.cooling_systems.add(clg_sys)

        return new_obj

    def ToString(self):
        """Overwrite .NET ToString."""
        return self.__repr__()

    def __repr__(self):
        """Properties representation."""
        return '{}'.format(self.__class__.__name__)



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Phius CO2-Emission Factor Library"""

# -- As per "Phius Core_Phius Zero_Final Modeling Protocol v1.1.pdf"
# -- https://www.phius.org/PHIUS+2021/Phius%20Core_Phius%20Zero_Final%20Modeling%20Protocol%20v1.1.pdf

factors_2021 = {
    "OIL": {"value": 309.9966, "unit": "g/kWh"},
    "NATURAL_GAS": {"value": 250.0171, "unit": "g/kWh"},
    "LPG": {"value": 270.0102, "unit": "g/kWh"},
    "HARD_COAL": {"value": 439.9864, "unit": "g/kWh"},
    "WOOD": {"value": 53.4289, "unit": "g/kWh"},
    "ELECTRICITY_MIX": {"value": 680.0068, "unit": "g/kWh"},
    "ELECTRICITY_PV": {"value": 250.0171, "unit": "g/kWh"},
    "HARD_COAL_CGS_70_CHP": {"value": 239.9864, "unit": "g/kWh"},
    "HARD_COAL_CGS_35_CHP": {"value": 319.9932, "unit": "g/kWh"},
    "HARD_COAL_CGS_0_CHP": {"value": 409.9966, "unit": "g/kWh"},
    "GAS_CGS_70_CHP": {"value": -70.0102, "unit": "g/kWh"},
    "GAS_CGS_35_CHP": {"value": 129.9898, "unit": "g/kWh"},
    "GAS_CGS_0_CHP": {"value": 319.9932, "unit": "g/kWh"},
    "OIL_CGS_70_CHP": {"value": 100, "unit": "g/kWh"},
    "OIL_CGS_35_CHP": {"value": 250.0171, "unit": "g/kWh"},
    "OIL_CGS_0_CHP": {"value": 409.9966, "unit": "g/kWh"},
}
